.TH Quaternion "" "June 11 1992" "COOL Manual" "Linear Algebra Classes"
.SH NAME
Quaternion \- Unit quaternion represents rotation in 3D.
.SH SYNOPSIS
#include <cool/Quaternion.h>
.SH DESCRIPTION
\f3Quaternion\f1 is a 4-element vector with 1 real and 3 imaginary components:
.DS
   q = r + (i*x + j*y + k*z) 
   r = cos(theta/2)
   (x, y, z) = sin(theta/2) (kx, ky, kz)
.DE
where theta and k are respectively the angle and axis of rotation. 
3D vectors can be thought of as imaginary quaternions, and so
a quaternion is represented as a \f3M_Vector<float>\f1 with the
imaginary part before the real part for 1-1 alignment.
.PP
Unit quaternion provides a more efficient representation for rotation, than
the usual orthonormal matrix that has nine parameters and six orthonormal 
constraints. The unit quaternion has only one unit magnitude constraint.
Composing rotations with quaternions results in fewer multiplications 
and less error. To insure valid rotation result, the nearest unit quaternion 
is computed, and this is much easier than finding the nearest orthonormal 
matrix. Transforming vectors with a quaternion requires more operations 
compared to multiplication with the equivalent orthonormal matrix.
.SH BASE CLASSES
\f3M_Vector<float>\f1.
.SH FRIEND CLASSES
None.
.SH CONSTRUCTORS
.TP
\f3Quaternion (float \f2x = 0\f3, float \f2y = 0\f3, float \f2z = 0\f3, float \f2r = 1\f3);\f1	
Creates a quaternion from its ordered components. \f2x, y, z\f1 denote
the imaginary part, which are the coordinates of the rotation axis multiplied
by the sine of half the angle of rotation. \f2r\f1 denotes the
real part, or the cosine of half the angle of rotation. Default is to
create a null quaternion, corresponding to a null rotation or an identity transform,
which has undefined rotation axis.
.TP
\f3Quaternion (const M_Vector<float>& \f2axis\f3, float \f2angle\f3);\f1
Creates a quaternion from the normalized \f2axis\f1 direction and the
\f2angle\f1 of rotation in radians.
.TP
\f3Quaternion (const Matrix<float>& \f2transform\f3);\f1
Creates a quaternion from a square \f2transform\f1 matrix, whose orthonormal
basis vectors are row-wise in the top-left most block. The \f2transform\f1
matrix can be 2x2, 3x3, or 4x4.
.TP
\f3inline Quaternion (const M_Vector<float>& \f2vector\f3);\f1
Creates a quaternion from a \f2vector\f1. 2D or 3D vector is converted
into an imaginary quaternion with same (x, y, z) components. 
4D vector is assumed to be a 4-element quaternion, to provide casting 
between \f3M_Vector\f1 and \f3Quaternion\f1.
.TP
\f3inline Quaternion (const Quaternion& \f2from\f3);\f1
Creates a copy of \f2from\f1 quaternion.
.TP
\f3inline ~Quaternion ();\f1
Frees space allocated for quaternion.
.SH MEMBER FUNCTIONS
.TP
\f3float angle () const;\f1
.TP
\f3M_Vector<float> axis () const;\f1
Queries the angle and the direction of the rotation axis of the quaternion.
A null quaternion will return zero angle and k axis.
.TP
\f3Quaternion conjugate () const;\f1
Returns the conjugate of given quaternion, having same real and opposite
imaginary parts.
.TP
\f3Quaternion inverse () const;\f1
Returns the inverse of given quaternion. For unit quaternion representing
rotation, the inverse is the same as the conjugate.
.TP	
\f3inline M_Vector<float> imaginary () const;\f1
.TP
\f3inline float real () const;\f1			
.TP
\f3inline float& x ();\f1
.TP
\f3inline float& y ();\f1
.TP
\f3inline float& z ();\f1
.TP
\f3inline float& r ();\f1	
Accessors for the imaginary and real components of the quaternion.
Use the last four accessors to both get and set the components.
.TP
\f3inline Quaternion& operator= (const Quaternion& \f2rhs\f1);\f1
Overloads assignment operator to copy \f2rhs\f1 quaternion into
\f2lhs\f1 quaternion.
.TP
\f3Boolean operator== (const Quaternion& \f2rhs\f1) const;\f1
.TP
\f3inline Boolean operator!= (const Quaternion& \f2rhs\f3) const;\f1
Two quaternions are equal if their four components are not different
by more than 1.0e-6.
.TP
\f3inline operator M_Vector<float>& ();\f1
Converts from a quaternion to a 4-element vector, through a type cast,
due to one-to-one mapping between quaternion and the 
coordinates (x,y,z,r).
.TP
\f3void rotate (M_Vector<float>& \f2v\f3) const;\f1
Rotates 3D vector \f2v\f1 with source quaternion and stores
the rotated vector back into \f2v\f1. For speed and greater accurary,
first convert quaternion into an orthonormal matrix, then use
matrix multiplication to rotate many vectors.
.TP
\f3Matrix<float> rotation_transform (int \f2dim = 4\f3) const;\f1
Returns the rotation transform as a square matrix with dimension \f2dim\f1.
This is the reverse counterpart of constructing a quaternion from a 
transformation matrix.
.SH FRIEND FUNCTIONS
.TP
\f3friend Quaternion operator* (const Quaternion& \f2q1\f3, const Quaternion& \f2q2\f3);\f1
Returns the product of two quaternions: \f2q1 * q2\f1.
Multiplication of two quaternions is not symmetric and has fewer operations 
than multiplication of orthonormal matrices. If object is rotated by \f2r1\f1, 
then by \f2r2\f1, then the composed rotation \f2(r2 o r1)\f1 is represented 
by the quaternion \f2(q2 * q1)\f1, or by the matrix \f2(m1 * m2)\f1. 
Note that matrix composition is reversed because matrices and vectors
are represented row-wise.
.TP
\f3inline friend ostream& operator<< (ostream& \f2os\f3, const Quaternion& \f2q\f3);\f1
.TP
\f3inline friend ostream& operator<< (ostream& \f2os\f3, const Quaternion* \f2q\f3);\f1
Prints a reference or a pointer to a quaternion \f2q\f1 to the output stream \f2os\f1.
.SH EXAMPLE
.DS
  #include <cool/Quaternion.h>
  #include <test.h>

  int main () {
    M_Vector<float> i(3,0.0), j(3,0.0), k(3,0.0);
    i.x() = j.y() = k.z() = 1.0;
    Quaternion q0(k, 0), q1(i, M_PI/2), 
    q2(j, M_PI/2), q3(k, M_PI/2);
    M_Vector<float> v;
    v = i; q0.rotate(v); 
    cout << v << endl;
    v = i; (q3*q2*q1).rotate(v); 
    cout << v << endl;
    v = i; q1.rotate(v); q2.rotate(v); q3.rotate(v); 
    cout << v << endl;
    (q3*q2*q1).conjugate().rotate(v); 
    cout << v << endl;
    return 0;
  }
.DE
.PP
has output:
.DS
  1 0 0 
  1.78814e-07 0 -1 
  3.55271e-15 5.96046e-08 -1 
  1 5.96046e-08 -2.38419e-07 
.DE
.SH SEE ALSO
\f3M_Vector<Type>\f1 and \f3Matrix<Type>\f1 for basic operations on vectors and
matrices.
.PP
\f3Transform\f1 for coordinate transformations.
.PP
\f3Envelope\f1 for envelope-letter scheme that avoids deep copy on return
by value in arithmetic expressions like: q1 * q2 * q3 *...
.SH COPYRIGHT
Copyright (C) 1992 General Electric Company.

Permission is granted to any individual or institution to use, copy, modify,
and distribute this software, provided that this complete copyright and
permission notice is maintained, intact, in all copies and supporting
documentation.

General Electric Company provides this software "as is" without
express or implied warranty.



